#! /bin/bash
args=($*)
export LC_ALL=en_US.UTF-8

# prevent root from running this script
if [ $EUID -eq 0 ] ; then
	echo "" 1>&2
	echo "ABORT! You tried to run this program on a non-Windows host as the \"root\" " 1>&2
	echo "user or the \"sudo\" command. This program runs many scripts created for " 1>&2
	echo "academic research. Some of those scripts can cause damage to a host when " 1>&2
	echo "run with \"root\" access rights. " 1>&2
	echo "" 1>&2
	echo "Therefore, this program has been disabled to run as \"root\". The manual " 1>&2
	echo "has more information about how to configure your Linux host to access your " 1>&2
	echo "data without running this program with \"root\" access. If you need more " 1>&2
	echo "help, please contact Precision Translation Tools via email at " 1>&2
	echo "sales@precisiontranslationtools.com for paid technical support options." 1>&2
	echo "" 1>&2
	exit 1
	fi

usage(){
	cat <<- HELP
	Usage: ${0##*/} [OPTION...]
	Run the second DoMY wizard
	
	Options:
	  -c       continue without pausing to read prompts
	  -s       skips $BASE train step
	  -t       set timer for training graphs
	  -r1      reuse micro corpus (train micro corpus only)
	  -r2      reuse domy corpus (train domy corpus only)
	  --clean  cleans demo output files
	  -h       this help text
	
	Examples:
	  time ${0##*/} -c -t
	     This command is useful to measure end-to-end performance.
	
	  ${0##*/} -s -c
	     This runs all data preparation and translation without
	     retraining the SMT models.
	
	HELP
	exit $1
	}

clean(){
	[ -d "$rootfolder/CORPORA/ready_micro/" ] && rm -rf "$rootfolder/CORPORA/ready_micro/"
	[ -d "$rootfolder/CORPORA/ready_micro-workbench/" ] && rm -rf "$rootfolder/CORPORA/ready_micro-workbench/"
	[ -d "$rootfolder/CORPORA/ready_domy/" ] && rm -rf "$rootfolder/CORPORA/ready_domy/"
	[ -d "$rootfolder/CORPORA/ready_domy-workbench/" ] && rm -rf "$rootfolder/CORPORA/ready_domy-workbench/"
#	[ -d "$rootfolder/BUILDS/lm/micro_lm/" ] && rm -rf "$rootfolder/BUILDS/lm/micro_lm/"
#	[ -d "$rootfolder/BUILDS/tm/micro_tm/" ] && rm -rf "$rootfolder/BUILDS/tm/micro_tm/"
#	[ -d "$rootfolder/BUILDS/lm/domy_lm/" ] && rm -rf "$rootfolder/BUILDS/lm/domy_lm/"
#	[ -d "$rootfolder/BUILDS/tm/domy_tm/" ] && rm -rf "$rootfolder/BUILDS/tm/domy_tm/"
	exit 0
	}

DOMY=$(dirname `readlink -f ${0}`)
BASE=domy

[[ "${args[@]}" =~ "-h" ]] && usage 0
[[ "${args[@]}" =~ "--help" ]] && usage 0

cfgm=${DOMY}/${BASE}
if [ ! -x "$cfgm" ] ; then
	echo "* $(date +%x\ %r) - ${0##*/} - installation corrupted - reinstall required"
	echo "$cfgm not found."
	exit 1
	fi
[ -f "$HOME/domy/$BASE.ini" ] && userfiles=$HOME/domy
[ -f "$HOME/domt/$BASE.ini" ] && userfiles=$HOME/domt

time_bin=`which time`
time_prefix=""
if [ -n "${time_bin}" ]; then
	[[ "${args[@]}" =~ "-t" ]] && time_prefix="${time_bin}"
	fi

# set graphs and rootfolder values using ini file if available
graphs=$userfiles/graphs-domy
plugins=$userfiles/plugins
rootfolder=$(dirname $(dirname ${DOMY}))/share/corpusfg
if [ ! -d "$(dirname $(dirname ${DOMY}))/share/corpusfg" ] ; then
	rootfolder=$(dirname $(dirname ${DOMY}))/share/domy
	fi

if [[ "${args[@]}" =~ "-clean" ]] ; then
	clean $*
	fi

if [ -f "$userfiles/$BASE.ini" ] ; then
	while read line ; do
		[[ "${line}" =~ "=" ]] && eval "${line// /}" 2>/dev/null
		done < "$userfiles/$BASE.ini"
	fi

graphsexist=0
deletegraphs=0
pluginsexist=0
deleteplugins=0
[ -d "${graphs}" ] && graphsexist=1
[ -d "${plugins}" ] && pluginsexist=1
if [ $graphsexist -eq 0 -o $pluginsexist -eq 0 ] ; then
	[[ "${args[@]}" =~ "-c" ]] && echo -e "* $(date +%x\ %r) - ${0##*/} - stop - can not run with -c option\nthe because \"${graphs##*/}\" or \"${plugins##*/}\" folder is missing. You can run \"${0##*/}\" \nwithout \"-c\", or run \"${BASE/.py/} -S\" to reset your configuration before\nrunning with the \"-c\" option." && exit 1
	"$cfgm" -vS
	if [ $? -ne 0 ] ; then echo "* $(date +%x\ %r) - ${0##*/} - configuration failure - reinstallation required" ; exit 1 ; fi
	fi
[ -d "${graphs}" -a ${graphsexist} -eq 0 ] && deletegraphs=1
[ -d "${plugins}" -a ${pluginsexist} -eq 0 ] && deleteplugins=1

leave(){
	[ ${deletegraphs} -eq 1 ] && rm -r "${graphs}"
	[ ${deleteplugins} -eq 1 ] && rm -r "${plugins}"
	[[ $1 -eq 255 ]] && echo
	exit $1
	}
trap "leave 255" SIGINT

clear

if [[ ! "${args[@]}" =~ "-r1" ]] ; then
if [[ ! "${args[@]}" =~ "-r2" ]] ; then
	echo "Introduction:"
	echo
	echo "This second demo runs example \"GRAPHS\" 6 to 14 to consolidate data,"
	echo "train two SMT models, evaluate SMT models and"
	echo "translate a document using both SMT models."
	echo
	echo "Example 6 cleans a micro corpus for use as SMT training corpus"
	echo "Example 7 creates the lm BUILD set named \"micro_lm\""
	echo "Example 8 creates the tm BUILD set named \"micro_tm\""
	echo "Example 9 trains, tunes, evaluates a \"micro\" SMT model"
	echo "Example 10 cleans a domy corpus from same data with different options"
	echo "Example 11 creates the lm BUILD set named \"domy_lm\""
	echo "Example 12 creates the tm BUILD set named \"domy_tm\""
	echo "Example 13 trains, tunes, evaluates a \"domy\" SMT model"
	echo "Example 14 translates sample document with both models"
	echo
	echo "You can find the GRAPHs and their config.ini files in:"
	echo "$graphs"
	echo

	echo "Next: Instructions"
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Instructions:"
	echo
	echo "This is a wizard-style demo. It pauses before running an example"
	echo "with a short description and the GRAPH's command line."
	echo
	echo "It pauses after running the example to show the command line,"
	echo "the input and output of the example. Right-click the input or"
	echo "output path and select \"Open Link\" to see the changes."
	echo

	echo "Next: Example 6, prepare nl-en sample data (micro set)"
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 6: prepare nl-en sample data (micro set)"
	echo
	echo "This GRAPH prepares sentence-aligned (sa) text files for use as SMT"
	echo "training corpus. Files in \"tm\" trees are processed as pairs. This"
	echo "GRAPH's config.ini file defines:"
	echo "     mintoken = 3"
	echo "     maxtoken = 20"
	echo "     ratio = 4:1"
	echo
	echo "Command:  ~\$ $BASE clean-tm"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 6..."
	clear




	echo "Example 6: prepare nl-en sample data (micro set)"
	echo
	echo "Command:  ~\$ $BASE clean-tm"
	echo
	if [ -f "$graphs/clean-tm/Demo - clean nl-en 'micro' parallel corpus.conf" ]; then
		cp "$graphs/clean-tm/config.ini" $graphs/clean-tm/.config.ini.tmp
		cp "$graphs/clean-tm/Demo - clean nl-en 'micro' parallel corpus.conf" $graphs/clean-tm/config.ini
		python $cfgm -v clean-tm
		result=$?
		mv $graphs/clean-tm/.config.ini.tmp $graphs/clean-tm/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE clean-tm"
		echo "Inputs:   \"file://$rootfolder/CORPORA/sa/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Outputs1: \"file://$rootfolder/CORPORA/ready_micro/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Outputs2: \"file://$rootfolder/CORPORA/ready_micro-workbench/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo
	else
		echo "\"clean-tm\" micro configuration is not installed."
		echo
		fi

	echo "Next: Example 7, create an lm BUILD set named \"micro_lm\""
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 7: create an lm BUILD set named \"micro_lm\""
	echo
	echo "Training an SMT model requires a language model \"BUILD\" set"
	echo "created. here, and a tm \"BUILD\" set created in Examples 10 & 11 below."
	echo "This GRAPH selects and consolidates target language data to create both"
	echo "a language model and to train a \"recaser\" model."
	echo
	echo "Command:  ~\$ $BASE build-lm"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 7..."
	clear


	echo "Example 7: create an lm BUILD set named \"micro_lm\""
	echo
	echo "Command:  ~\$ $BASE build-lm"
	echo
	if [ -f "$graphs/build-lm/Demo - build nl-en 'LM BUILD set' with 'micro' corpus.conf" ]; then
		cp "$graphs/build-lm/config.ini" $graphs/build-lm/.config.ini.tmp
		cp "$graphs/build-lm/Demo - build nl-en 'LM BUILD set' with 'micro' corpus.conf" $graphs/build-lm/config.ini
		python $cfgm -v build-lm
		result=$?
		mv $graphs/build-lm/.config.ini.tmp $graphs/build-lm/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE build-lm"
		echo "Inputs1:  \"file://$rootfolder/CORPORA/ready_domy/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Inputs2:  \"file://$rootfolder/CORPORA/ready_domy-workbench/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Inputs3:  \"file://$rootfolder/CORPORA/ready_domy-workbench/EuroParl%20v5/Legislation/debates/lm/nl/\""
		echo "Output:   \"file://$rootfolder/BUILDS/lm/micro_lm/\""
		echo
	else
		echo "\"build-lm\" micro configuration is not installed."
		echo
		fi

	echo "Next: Example 8, create a tm BUILD set named \"micro_tm\""
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 8: create a tm BUILD set named \"micro_tm\""
	echo
	echo "The example GRAPHS clean-tm processed all data in their original,"
	echo "separate files. This allows users to maintain data categories during"
	echo "processing. The separate data files must be consolidated before"
	echo "training. The \"BUILD\" process selects, consolidates the data. This"
	echo "GRAPH creates a tm \"BUILD\" set with 12 files from the \"micro\""
	echo "SMT data set created by clean-tm. This GRAPH's config file defines:"
	echo "     mertset = 300"
	echo "     evalset = 300"
	echo
	echo "Command:  ~\$ $BASE build-tm"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 8..."
	clear




	echo "Example 8: create a tm BUILD set named \"micro_tm\""
	echo
	echo "Command:  ~\$ $BASE build-tm"
	echo
	if [ -f "$graphs/build-tm/Demo - build nl-en 'TM BUILD set' with 'micro' parallel corpus.conf" ]; then
		cp "$graphs/build-tm/config.ini" $graphs/build-tm/.config.ini.tmp
		cp "$graphs/build-tm/Demo - build nl-en 'TM BUILD set' with 'micro' parallel corpus.conf" $graphs/build-tm/config.ini
		python $cfgm -v build-tm
		result=$?
		mv $graphs/build-tm/.config.ini.tmp $graphs/build-tm/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE build-tm"
		echo "Inputs:   \"file://$rootfolder/CORPORA/ready_domy/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Outputs:  \"file://$rootfolder/BUILDS/tm/micro_tm\""
		echo
	else
		echo "\"build-tm\" micro configuration is not installed."
		echo
		fi

	echo "Next: Example 9, train an SMT model with BUILD sets named \"micro_tm\" and \"micro_lm\""
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 9: train an SMT model with BUILD sets named"
	echo "            \"micro_tm\" and \"micro_lm\""
	echo
	echo "The training process consists of five sub-steps:"
	echo "   1) train-lm      - trains the language model"
	echo "   2) train-tm      - trains the phrase and reordering tables, then"
	echo "                      binarizes the tables This takes a long time."
	echo "   3) train-mert    - \"Minimum Error Rate Tuning\" creates an SMT"
	echo "                      model consisting of phrase and reordering tables,"
	echo "                      a language model and configuration file with"
	echo "                      optimal settings. This takes a long time."
	echo "   4) train-eval    - evaluates the \"tuned\" SMT model"
	echo "   5) train-recaser - trains a recaser model that restores"
	echo "                      upper/lower case to translations"
	echo
	echo "Command:  ~\$ $BASE train"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 9..."
	clear
	fi
	fi



if [[ ! "${args[@]}" =~ "-r2" ]] ; then
	echo "Example 9: train an SMT model with BUILD sets named"
	echo "            \"micro_tm\" and \"micro_lm\""
	echo
	echo "Command:  ~\$ $BASE train"
	echo
	if [ -f "$graphs/train/Demo - train, tune, eval nl-en SMT model with 'micro' corpus.conf" ]; then

		if [[ ! "${args[@]}" =~ "-s" ]] ; then
			cp "$graphs/train/config.ini" $graphs/train/.config.ini.tmp
			cp "$graphs/train/Demo - train, tune, eval nl-en SMT model with 'micro' corpus.conf" $graphs/train/config.ini
			[ -n "${time_prefix}" ] && time_prefix="${time_bin} -po $userfiles/time-micro.log "
			${time_prefix} python $cfgm -v train
			result=$?
			mv $graphs/train/.config.ini.tmp $graphs/train/config.ini
			[ $result -ne 0 ] && leave $result
			echo
			echo "Review"
			echo "Command:  ~\$ $BASE train"
			echo "Input1:   \"file://$rootfolder/BUILDS/lm/micro_lm\""
			echo "Input2:   \"file://$rootfolder/BUILDS/tm/micro_tm\""
			echo "Output:  \"file://$rootfolder/ENGINES/evals/eval-s=nl-t=en-p=micro_tm-a=giza-g=3-l=micro_lm-T=irstlm-n=3/moses-mert.mteval-v12.pl.txt\""
			echo
			fi
		[[ "${args[@]}" =~ "-r1" ]] && leave 0
	else
		echo "\"train\" micro configuration is not installed."
		echo
		fi
	fi

if [[ ! "${args[@]}" =~ "-r1" ]] ; then
if [[ ! "${args[@]}" =~ "-r2" ]] ; then
	echo "Next: Example 10, prepare nl-en sample data (domy set)"
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 10: prepare nl-en sample data (domy set)"
	echo
	echo "This GRAPH starts with the same data as Example 6. New \"mintoken\","
	echo "and \"maxtoken\" settings create a larger SMT training data set. This"
	echo "GRAPH's config file defines:"
	echo "     mintoken = 2"
	echo "     maxtoken = 65"
	echo "     ratio = 4:1"
	echo
	echo "Command:  ~\$ $BASE clean-tm"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 10..."
	clear




	echo "Example 10: prepare nl-en sample data (domy set)"
	echo
	echo "Command:  ~\$ $BASE clean-tm"
	echo
	if [ -f "$graphs/clean-tm/Demo - clean nl-en 'domy' parallel corpus.conf" ]; then
		cp "$graphs/clean-tm/config.ini" $graphs/clean-tm/.config.ini.tmp
		cp "$graphs/clean-tm/Demo - clean nl-en 'domy' parallel corpus.conf" $graphs/clean-tm/config.ini
		python $cfgm -v clean-tm
		result=$?
		mv $graphs/clean-tm/.config.ini.tmp $graphs/clean-tm/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE clean-tm"
		echo "Inputs:   \"file://$rootfolder/CORPORA/sa/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Outputs1: \"file://$rootfolder/CORPORA/ready_domy/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Outputs2: \"file://$rootfolder/CORPORA/ready_domy-workbench/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo
	else
		echo "\"clean-tm\" domy configuration is not installed."
		echo
		fi

	echo "Next: Example 11, create an lm BUILD set named \"domy_lm\""
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 11: create an lm BUILD set named \"domy_lm\""
	echo
	echo "Training an SMT model requires a language model \"BUILD\" set"
	echo "created. here, and a tm \"BUILD\" set created in Examples 10 & 11 below."
	echo "This GRAPH selects and consolidates target language data to create both"
	echo "a language model and to train a \"recaser\" model."
	echo
	echo "Command:  ~\$ $BASE build-lm"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 11..."
	clear


	echo "Example 11: create an lm BUILD set named \"domy_lm\""
	echo
	echo "Command:  ~\$ $BASE build-lm"
	echo
	if [ -f "$graphs/build-lm/Demo - build nl-en 'LM BUILD set' with 'domy' corpus.conf" ]; then
		cp "$graphs/build-lm/config.ini" $graphs/build-lm/.config.ini.tmp
		cp "$graphs/build-lm/Demo - build nl-en 'LM BUILD set' with 'domy' corpus.conf" $graphs/build-lm/config.ini
		python $cfgm -v build-lm
		result=$?
		mv $graphs/build-lm/.config.ini.tmp $graphs/build-lm/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE build-lm"
		echo "Inputs1:  \"file://$rootfolder/CORPORA/ready_domy/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Inputs2:  \"file://$rootfolder/CORPORA/ready_domy-workbench/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Inputs3:  \"file://$rootfolder/CORPORA/ready_domy-workbench/EuroParl%20v5/Legislation/debates/lm/nl/\""
		echo "Output:   \"file://$rootfolder/BUILDS/lm/domy_lm/\""
		echo
	else
		echo "\"build-lm\" domy configuration is not installed."
		echo
		fi

	echo "Next: Example 12, create a tm BUILD set named \"domy_tm\""
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 12: create a tm BUILD set named \"domy_tm\""
	echo
	echo "This GRAPH selectes and consolidates the larger SMT data set created"
	echo "in clean-tm and creates a tm \"BUILD\" set of 12 files. This"
	echo "GRAPH's config file defines:"
	echo "     mertset = 500"
	echo "     evalset = 500"
	echo
	echo "Command:  ~\$ $BASE build-tm"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 12..."
	clear







	echo "Example 12: create a tm BUILD set named \"domy_tm\""
	echo
	echo "Command:  ~\$ $BASE build-tm"
	echo
	if [ -f "$graphs/build-tm/Demo - build nl-en 'TM BUILD set' with 'domy' parallel corpus.conf" ]; then
		cp "$graphs/build-tm/config.ini" $graphs/build-tm/.config.ini.tmp
		cp "$graphs/build-tm/Demo - build nl-en 'TM BUILD set' with 'domy' parallel corpus.conf" $graphs/build-tm/config.ini
		python $cfgm -v build-tm
		result=$?
		mv $graphs/build-tm/.config.ini.tmp $graphs/build-tm/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE build-tm"
		echo "Inputs:   \"file://$rootfolder/CORPORA/ready_domy/EuroParl%20v5/Legislation/debates/tm/nl/\""
		echo "Outputs:  \"file://$rootfolder/BUILDS/tm/domy_tm\""
		echo
	else
		echo "\"build-tm\" domy configuration is not installed."
		echo
		fi

	echo "Next: Example 13, train an SMT model with BUILD sets named \"domy_tm\" and \"domy_lm\""
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear

	echo "Example 13: train an SMT model with BUILD sets named"
	echo "            \"domy_tm\" and \"domy_lm\""
	echo
	echo "The training process consists of five sub-steps:"
	echo "   1) train-lm      - trains the language model"
	echo "   2) train-tm      - trains the phrase and reordering tables, then"
	echo "                      binarizes the tables This takes a long time."
	echo "   3) train-mert    - \"Minimum Error Rate Tuning\" creates a translation"
	echo "                      model consisting of phrase and reordering tables,"
	echo "                      a language model and configuration file with"
	echo "                      optimal settings. This takes a long time."
	echo "   4) train-eval    - evaluates the \"tuned\" SMT model"
	echo "   5) train-recaser - trains a recaser model that restores"
	echo "                      upper/lower case to translations"



	echo
	echo "Command:  ~\$ $BASE train"
	echo
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 13..."
	clear
	fi
	fi

if [[ ! "${args[@]}" =~ "-r1" ]] ; then
	echo "Example 13: train an SMT model with BUILD sets named"
	echo "            \"domy_tm\" and \"domy_lm\""
	echo
	echo "Command:  ~\$ $BASE train"
	echo
	if [ -f "$graphs/train/Demo - train, tune, eval nl-en SMT model with 'domy' corpus.conf" ]; then

		if [[ ! "${args[@]}" =~ "-s" ]] ; then
			cp "$graphs/train/config.ini" $graphs/train/.config.ini.tmp
			cp "$graphs/train/Demo - train, tune, eval nl-en SMT model with 'domy' corpus.conf" $graphs/train/config.ini
			[ -n "${time_prefix}" ] && time_prefix="${time_bin} -po $userfiles/time-domy.log "
			${time_prefix} python $cfgm -v train
			result=$?
			mv $graphs/train/.config.ini.tmp $graphs/train/config.ini
			[ $result -ne 0 ] && leave $result
			echo
			echo "Review"
			echo "Command:  ~\$ $BASE train"
			echo "Input1:   \"file://$rootfolder/BUILDS/lm/domy_lm\""
			echo "Input2:   \"file://$rootfolder/BUILDS/tm/domy_tm\""
			echo "Output:  \"file://$rootfolder/ENGINES/evals/eval-s=nl-t=en-p=domy_tm-a=giza-g=3-l=domy_lm-T=irstlm-n=3/moses-mert.mteval-v12.pl.txt\""
			echo
			fi
		[[ "${args[@]}" =~ "-r2" ]] && leave 0
	else
		echo "\"train\" domy configuration is not installed."
		echo
		fi
	fi


echo "Next: Example 14, translate sample document"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 14: translate sample document"
echo
echo "This example uses the \"translate\" GRAPH to translate a sample"
echo "document using the \"domy_tm\" translation engine."
echo
echo "Command:  ~\$ $BASE translate"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 14..."
clear

echo "Example 14: translate sample document"
echo
echo "Command:  ~\$ $BASE translate"
echo
if [ -f "$graphs/translate/Demo - translate nl-en with 'micro' and 'domy' SMT and recaser models.ini" ]; then


	if [[ ! "${args[@]}" =~ "-s" ]] ; then
		cp "$graphs/translate/config.ini" $graphs/translate/.config.ini.tmp
		cp "$graphs/translate/Demo - translate nl-en with 'micro' and 'domy' SMT and recaser models.ini" $graphs/translate/config.ini
		python $cfgm -v translate
		result=$?
		mv $graphs/translate/.config.ini.tmp $graphs/translate/config.ini
		[ $result -ne 0 ] && leave $result
		echo
		echo "Review"
		echo "Command:  ~\$ $BASE translate"
		echo "Inputs:   \"file://$rootfolder/TRANSLATIONS/in/lm/nl/nl/sample-ce.txt\""
		echo "Outputs:  \"file://$rootfolder/TRANSLATIONS/out/tm/nl/sample-ce.txt.tmx\""
		echo
		fi
else
	echo "\"translate\" domy configuration is not installed."
	echo
	fi

echo "Next: Review"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Review:"
echo
echo "Demo-2.sh examples created \"BUILD\" sets, trained translation"
echo "engines and translated an example document."
echo
echo "Example 6 cleaned a micro corpus for use as SMT training corpus"
echo "Example 7 created the lm BUILD set named \"micro_lm\""
echo "Example 8 created the tm BUILD set named \"micro_tm\""
echo "Example 9 trained, tuned, evaluated a \"micro\" SMT model"
echo "Example 10 cleaned a domy corpus from same data with different options"
echo "Example 11 created the lm BUILD set named \"domy_lm\""
echo "Example 12 created the tm BUILD set named \"domy_tm\""
echo "Example 13 trained, tune, evaluate a \"domy\" SMT model"
echo "Example 14 translated sample document with an engine using both models"

echo "Next: NOTES"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "NOTE: All outputs from these examples were saved in sub-folders"
echo "  under \"$rootfolder/BUILDS/\", \"$rootfolder/ENGINES/\", and"
echo "  \"$rootfolder/TRANSLATIONS/\". To delete these demo files after"
echo "  running \"demo-2.sh\" in DoMY CE, run \"demo-2.sh -clean\"."
echo
echo "NOTE 2: You may want to rerun demo-2.sh. If you first delete the"
echo "  output folders, you can watch the demo create the output files"
echo "  and review the results of each example before proceeding to"
echo "  the next example. If you do not delete the output, the sub-steps"
echo "  in \"train\" examples will detect the previous output and skip to"
echo "  the next step."
echo

echo "Next: What's next?"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Next Steps:"
echo
echo "1) review the user's manual \"DoMY™ CE Users Manual.doc\""
echo "   from the icon on your desktop, or in your $graphs"
echo "   folder on server hosts"
echo "2) pick an example GRAPH that best matches your needs. Edit the"
echo "   config.ini file ($graphs) to your language pair."
echo "3) put your data files in a folder hierarchy folder under"
echo "   $rootfolder/CORPORA that matches your needs."
echo "4) run the edited GRAPH configuration."
echo

leave 0

# DoMY™ CE v2.5.339
# Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

# For more information, please contact Precision Translation Tools Co., Ltd.
# at: http://www.precisiontranslationtools.com
