#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Walapa Muangjeen
#===============================================================================

#version:
#2.5.343 - added escapes to the _nobreaks list

import os
import sys
import common as cf
import logging
import re

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

_nobreaks = (
	re.compile(r'(,)'),
	re.compile(r'(\.)'),
	re.compile(r'(\?)'),
	re.compile(r'(!)'),
	re.compile(r'(\\n)'),
	re.compile(r'({\d+})'),
	re.compile(r'(<\d+>)'),
	re.compile(r'(</\d+>)'),
	re.compile(r'(<\d+/>)'),
	re.compile(r'(&amp; *[ a-zA-Z0-9]+ *;)'),
	re.compile(r'(&amp; *# *[ 0-9]+ *;)'),
	re.compile(r'(&amp; *# *[xXuU] *[ a-zA-Z0-9]+ *;)'),
	re.compile(r'(& +[ a-zA-Z0-9]+ *;)'),
	re.compile(r'(& *# *[ 0-9]+ *;)'),
	re.compile(r'(& *# *[xXuU] *[ a-zA-Z0-9]+ *;)'),
	re.compile(r'(% *\( *[ a-zA-Z0-9]+ *\) *s)'),
	re.compile(r'(% *\( *#+ *[ 0-9]+ *\) *s)'),
	re.compile(r'(% *\( *#+ *[xXuU] *[ a-zA-Z0-9]+ *\) *s)'),
	re.compile(r'(\$ *\( *[ a-zA-Z0-9_]+ *\))'),
	re.compile(r'(\$ *{ *[ a-zA-Z0-9_]+ *})'),
	)

class filter(object):

	cfg = {
		'encoding': 'utf8',
		'inputfiles': '',
		'outputfiles': '',
		'by-character': False,
		'remove-maxtokens': False,
		'mintokens': 1,
		'maxtokens': 65,
		'version': '2.5.343',
		}
	encoding = 'utf8'
	inputfiles = []
	outputfiles = []
	by_character = False
	remove_maxtokens = False
	mintokens = 1
	maxtokens = 50
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.inputfiles = cfg['inputfiles'].split(';') if cfg['inputfiles'] else self.inputfiles
		self.outputfiles = cfg['outputfiles'].split(';') if cfg['outputfiles'] else self.outputfiles

		if len(self.inputfiles) != len(self.outputfiles):
			self.errors.append([__name__,'invalid','[%s] inputfiles=%s mismatch outputfiles= value'%(__name__,cfg['inputfiles'])])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.by_character = cfg['by-character']
		self.remove_maxtokens = cfg['remove-maxtokens']
		self.maxtokens = cfg['maxtokens'] if cf.isinteger(cfg['maxtokens']) and cfg['maxtokens'] <= 100 else self.maxtokens
		self.mintokens = cfg['mintokens'] if cf.isinteger(cfg['mintokens']) and 1 <= cfg['mintokens'] <= self.maxtokens else self.mintokens
		if not self.maxtokens > self.mintokens:
			self.errors.append([__name__,'invalid','[%s] maxtokens=%s value must be greater than mintokens=%s'%(__name__,self.maxtokens,self.mintokens)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

	def run(self,k):
		global skipclose
		skipclose = not self.inputfiles
		if self.inputfiles: return

		if self.remove_maxtokens:

			for linenum, line in enumerate(self.p.cfoutput[k]['tempbuff']):
				line = line.rstrip('\r\n')

				keep = _keep(line,self.mintokens,self.maxtokens)
				if keep:
					self.p.cfoutput[k]['tempbuff'][linenum] = line

		else:
			if self.by_character:
				for linenum,line in enumerate(self.p.cfoutput[k]['tempbuff']):
					line = _filter(line,self.maxtokens,self.mintokens)
					if line:
						while line[-1] == '\x0a':
							line = line[:-1]
						self.p.cfoutput[k]['tempbuff'][linenum] = '%s\x0a'%('\x0a'.join(line))
			else:
				for linenum,line in enumerate(self.p.cfoutput[k]['tempbuff']):
					j = 0
					line = line.split()
					line = [' '.join(line[j:j+self.maxtokens]) for j in range(j,len(line),self.maxtokens) if len(line[j:j+self.maxtokens]) >= self.mintokens]
					if line:
						while line[-1] == '':
							line = line[:-1]
						self.p.cfoutput[k]['tempbuff'][linenum] = '%s\x0a'%('\x0a'.join(line))

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		import codecs
		from progress_bar import ProgressBar
		import shutil

		for index, filename in enumerate(self.inputfiles):
			if not os.path.exists(filename):
				self.errors.append([__name__,'missing','[%s] %s'%(__name__,filename)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				return

		try:

			for index,filename in enumerate(self.outputfiles):

				# make output folder
				try:
					os.makedirs(os.path.dirname(self.outputfiles[index]))
				except OSError,e:
					if not e.errno == 17:
						logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
						raise OSError(e)

				# open inputs and outputs files
				out = self.outputfiles[index]
				if out == self.inputfiles[index]:
					import tempfile
					fd,out = tempfile.mkstemp(suffix='.tmp', prefix='~', dir=self.p.tempdir)
					os.close(fd)
				try:
					o = codecs.open(out,'w',self.encoding)
					i = codecs.open(self.inputfiles[index],'r',self.encoding)
					ttl_lines = cf.file_length(self.inputfiles[index])
				except:
					raise RuntimeError('Failed to open [%s] input/output files'%(__name__))

				progress = ProgressBar()
				progress.reset(0,ttl_lines,72,mode='fixed',char='#')
				sys.stderr.write('[%s] %s\n'%(__name__,self.outputfiles[index].encode('utf8')))

				# loop writes output line-by-line
				if self.remove_maxtokens:

					for line in i:
						line = line.rstrip('\r\n')

						keep = _keep(line,self.mintokens,self.maxtokens)
						if keep:
							o.write('%s\x0a'%(line))

						progress.increment_amount()
						sys.stderr.write(progress.bar)
						sys.stderr.flush()

				else:

					if self.by_character:

						for line in i:
							line = line.rstrip('\r\n')
							line = _filter(line,self.maxtokens,self.mintokens)
							if line:
								while line[-1] == '\x0a':
									line = line[:-1]
								o.write('%s\x0a'%(line))

							progress.increment_amount()
							sys.stderr.write(progress.bar)
							sys.stderr.flush()

					else:
						for line in i:
							j = 0
							line = line.split()
							line = [' '.join(line[j:j+self.maxtokens]).strip() for j in range(j,len(line),self.maxtokens) if len(line[j:j+self.maxtokens]) >= self.mintokens]
							if line:
								while line[-1] == '':
									line = line[:-1]
								o.write('%s\x0a'%('\x0a'.join(line)))

							progress.increment_amount()
							sys.stderr.write(progress.bar)
							sys.stderr.flush()

				if not out == self.outputfiles[index]:
					import shutil

				# close input and output files
				i.close()
				o.close()
				sys.stderr.write('\n')
				progress.update_amount(ttl_lines)
				sys.stderr.write(progress.bar)
				sys.stderr.write('\n')
				sys.stderr.flush()

				if not out == self.outputfiles[index]:
					shutil.move(out,self.outputfiles[index])

				if not os.path.exists(self.outputfiles[index]):
					self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.outputfiles[index])])
					logger.error('%s\t%s',*self.errors[-1][1:])

		except KeyboardInterrupt:
			os.unlink(out)
			raise KeyboardInterrupt()

def _filter(line,maxtokens,mintokens):
	newline = []
	prefix = ''

	i = 0
	while len(line.strip()):
		found = False
		for p in _nobreaks:
			for m in p.finditer(line):
				if m.start() < maxtokens-len(prefix) < m.end():
					if len(line[:m.start()]) >= mintokens:
						newline.append('%s%s'%(prefix,line[:m.start()]))
					line = line[m.end():]
					prefix = m.group()
					found = True
					break
			if found: break
		if not found:
			line = '%s%s'%(prefix,line)
			if len(line[:maxtokens]) >= mintokens: 
				newline.append(line[:maxtokens])
			line = line[maxtokens:]
			prefix = ''

	return '\x0a'.join(newline)

def _keep(line,mintokens,maxtokens):
	leng = len(line.split())
	keep = (mintokens <= leng <= maxtokens)
#	keep = keep and 0.2 <= (leng / (leng + leng)) <= 0.8


def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
